% RIM from geometrical optics point of view is demonstrated as shown in figure 1
% sample is above, objective is below
% normal rays, virtual rays, real wavefronts, ideal wavefronts and caustics
% are plotted
clear
%% RIM geometry
%immersion refractive index
n1=1.33;
% sample refractive index
n2=1.00;
% Numerical Aperture 
NA=min(min(0.95,0.95*n1),n2);
% fluorophore position
z_f=10; 
 % interface position (z_A=z_f-z_i)
z_i=0; 
%% Visualisation parameters
% rays are drawn until this axial coordinate
z_end=-5*(z_f-z_i)+z_i;
% lateral scale
y_end=3*(z_f-z_i);

%(half)-number of rays
N=7;
% ray visuals
ray_col='k';ray_line_style='-';ray_width=3;
% virtual pre-interface rays visuals
v_ray_col='r';v_ray_line_style='--';v_ray_width=2;

%number points on the wavefronts and caustics
N_w=100;
% wavefront correspond to accumulated optical paths of
ph=0.4*(z_f-z_i)*n2*[1.5 2 3.5 4 5.5 6 ];
% wavefronts visuals
wave_col='b';wave_line_style='-';wave_width=3;
% ideal wavefronts visuals
i_wave_col='b';i_wave_line_style='--';i_wave_width=2;
% caustics visuals
c_col='g';c_line_style='-';c_width=3;
%% Geometrical Optics
% Definitions
% ray angles
theta=asin(NA/n2)*(0:N-1)/(N-1);
% wavefront angles
% actual wavefronts
phi=asin(NA/n2)*((0:N_w-1)/(N_w-1));
% ideal spherical wavefronts
phi_ideal=pi/2*((0:N_w-1)/(N_w-1));

% intersection with the interface   
% rays hitting interface at a lateral positon of
y0=(z_f-z_i)*tan(theta);
% wavefronts hitting interface at a lateral positon of
y0_w=(z_f-z_i)*tan(phi);
% wavefronts accumulated this much optical length reaching the interface
l_1=n2*(z_f-z_i)./cos(phi);

% Snell's law applied:

% angles of rays after refraction
theta2=(asin(n2/n1*sin(theta)));
% finding first ray that underwent Total Internal Reflection
ind_TIR=find(imag(theta2)~=0,1,'first');
% rays after Total Internal Reflection are not shown
theta2(ind_TIR:end)=NaN;

% angles of wavefront propagation after refraction; the rest of GO 
% for wavefronts is done together with drawing in the end of the file
phi2=(asin(n2/n1*sin(phi)));
% finding first wavefront point that underwent Total Internal Reflection
ind_TIR=find(imag(phi2)~=0,1,'first');
% rays after Total Internal Reflection are not shown
phi2(ind_TIR:end)=NaN;

% finding caustics
% two shifted indices of wavefront points
ind_1=1:(N_w-1);
ind_2=2:N_w;
% redefining propagation as y=k*z+b
k=-tan(phi2);
b=y0_w-k*z_i;
% finding intersections of neigbohring wavefront points
% axial coordinate
z_c = (b(ind_2)-b(ind_1))./(k(ind_1)-k(ind_2));
% lateral coordinate
y_c = k(ind_1).*z_c+b(ind_1);
%% Drawing of the figure starts
figure(1);clf;hold on;grid on;
%% Drawing the interface
plot ( y_end*[-1, 1],[z_i, z_i],'k','Marker','none','LineWidth',2,'LineStyle','--');
%% Drawing rays pre-interface
% on the right side
plot([0*ones(N,1) y0']',[z_f*ones(N,1) z_i*ones(N,1)]', ... 
    ray_col,'LineStyle',ray_line_style,'LineWidth',ray_width);
% on the left side
plot(-[0*ones(N,1) y0']',[z_f*ones(N,1) z_i*ones(N,1)]',... 
    ray_col,'LineStyle',ray_line_style,'LineWidth',ray_width);
%% Drawing rays post-interface
% on the right side
plot([y0' (y0-tan(theta2)*(z_end-z_i))']',[z_i*ones(N,1) z_end*ones(N,1)]', ... top
    ray_col,'LineStyle',ray_line_style,'LineWidth',ray_width);% top
% on the left side
plot(-[y0' (y0-tan(theta2)*(z_end-z_i))']',[z_i*ones(N,1) z_end*ones(N,1)]', ... bottom
    ray_col,'LineStyle',ray_line_style,'LineWidth',ray_width);%bottom
%% Drawing virtual pre-interface rays
% on the left side
plot([y0' (y0-tan(theta2)*(-z_end-z_i))']',[z_i*ones(N,1) -z_end*ones(N,1)]', ...
    v_ray_col,'LineStyle',v_ray_line_style,'LineWidth',v_ray_width);
% on the right side
plot( -[y0' (y0-tan(theta2)*(-z_end-z_i))']',[z_i*ones(N,1) -z_end*ones(N,1)]',...
    v_ray_col,'LineStyle',v_ray_line_style,'LineWidth',v_ray_width);
%% Drawing caustics
plot(y_c,z_c,c_col,'LineWidth',c_width,'LineStyle',c_line_style)
plot(-y_c,z_c,c_col,'LineWidth',c_width,'LineStyle',c_line_style)

%% Drawing wavefronts at each pathlength specified
 % some GO is done here also
for i=1:numel(ph)
% ideal wavefront propagation without interface: 
z_l=z_f-ph(i)/n2*cos(phi);
y_l=ph(i)/n2*sin(phi);
%  the parts of the wavefront that crossed the interface
ind=find(ph(i)>=l_1);
% propagation after the interface in the n1 medium:
z_l(ind)=z_i-      (ph(i)-l_1(ind))/n1.*cos(phi2(ind));
y_l(ind)=y0_w(ind)+(ph(i)-l_1(ind))/n1.*sin(phi2(ind));
%% ideal wavefronts
% finding a sphere that coincides with wavefront on axis
% center of sphere is at
z_sphere=(z_l(1)^2+y_l(1)^2-z_l(2)^2-y_l(2)^2)/(2*(z_l(1)-z_l(2)));
% radius
R_sphere=((z_l(1)-z_sphere)^2+y_l(1)^2)^0.5;

% right half
plot(R_sphere*sin(phi_ideal),z_sphere-R_sphere*cos(phi_ideal),...
    i_wave_col,'Marker','none','LineWidth',i_wave_width,'LineStyle',i_wave_line_style);
% left half
plot(-R_sphere*sin(phi_ideal),z_sphere-R_sphere*cos(phi_ideal),...
    i_wave_col,'Marker','none','LineWidth',i_wave_width,'LineStyle',i_wave_line_style);
%% real wavefronts
% right half
plot(real(y_l),real(z_l),... top
    wave_col,'Marker','none','LineWidth',wave_width,'LineStyle',wave_line_style);
% left half
plot(-real(y_l),real(z_l),... bottom
    wave_col,'Marker','none','LineWidth',wave_width,'LineStyle',wave_line_style);

end

%% boundaries
axis equal;
ylim(min(abs(z_end),abs(y_end))*[-1 1]);
xlim(z_i+min(abs(z_end),abs(y_end))*[-1 1]);
set(gca,'FontSize',20);
xlabel('Y, \mum');ylabel('Z, \mum')